<?php

/**
 * Get emails from members of a forum
 * $uid is the user posting, depending on the user's email copy setting, may be excluded from results
 */
function _femail_get_emails($tid, $uid, $digest = 1, $additional_emails = TRUE){
  // Find out if user option is set to receive copy of own emails.
  $user_email_copy = _femail_load_user_copy($uid);
  // Select all active users, including $uid if $user_email_copy == TRUE
  $results = _femail_select_active_users($user_email_copy, $uid);
  // Get uids of active users
  $uids = array();
  foreach($results as $result){
    $uids[] = $result->uid;
  }
  // Load full users objects for active users
  $users = user_load_multiple($uids);
  $emails = array();
  foreach($users as $account){
    // Check User Forum Access
    $user_access = forum_access_access('create', $tid, $account);
    // Check User Digest Option
    $user_digest = _femail_check_user_digest_option($account->uid, $tid);
    if ($user_digest == NULL){
      $user_digest = 1;
    }
    // Check Forum Override
    $forum_override = _femail_check_forum_override($tid);
    if ($forum_override == NULL){
      $forum_override = -1;
    }

    if (
      ($user_access)
      &&
      ($forum_override != 0)
      &&
      (
        ($forum_override == $digest)
        ||
        (
          ($user_digest == $digest)
          &&
          ($forum_override == -1)
        )
      )
    ){
      $emails[] = "<{$account->mail}>";
      if($additional_emails){
        //get additional verified, enabled emails and add to list of emails to send to
        $additional_emails = _femail_load_additional_verified_emails($account->uid);
        foreach($additional_emails as $additional_email){
          $emails[] = "<{$additional_email->email}>";
        }
      }
    }
  }
  return $emails;
}

/** Get all emails for all forums for a $digest level */
function _femail_get_all_emails($digest = 1){
  // Iterate through all forums
  $query = db_select('forum_index', 'f');
  $query->fields('f', array('tid'));
  $query->distinct();
  $result = $query->execute();
  $emails = array();
  foreach($result as $record) {
    $tid = $record->tid;
    // Don't get additional emails now only account emails
    $emails = array_unique(array_merge($emails, _femail_get_emails($tid, 0, $digest, FALSE)), SORT_REGULAR);
  }
  foreach($emails as &$email){
    $email = preg_replace('/[<>]/', '', $email);
  }
  return $emails;
}

function _femail_send_message($tid, $subject, $body, $nid, $files = array(), $cid = 0, $inreplyto = FALSE){
  global $user;

  $emails = _femail_get_emails($tid, $user->uid);
  $forum_term = taxonomy_term_load($tid);
  if(count($emails)){
    $from_emails = variable_get('femail_emails', array());
    $from = ($user->name ? '=?UTF-8?B?' . base64_encode($user->name) . '?= ' : '') . '<' . ($user->mail ? $user->mail : $from_emails[$tid]) . '>';
    global $base_url;
    $parts = parse_url($base_url);
    $msgid = '<' . md5(microtime()) . '@' . $parts['host'] . '>';
    $node = node_load($nid);
    db_insert('femail_msgs')->fields(array(
      'nid',
      'cid',
      'msgid'
    ), array(
      $nid,
      $cid,
      $msgid
    ))->execute();
    if ($cid) {
      $comment_text = '#comment-' . $cid;
    } else {
      $comment_text = "";
    }

    // Filtered HTML
    $body = check_markup($body, "filtered_html");

    // Delete Signatures
    $sigpos = strpos($body, "femail-signature");
    if ($sigpos >= 12) {
      $sigpos = $sigpos - 12;
      $body = substr($body, 0, $sigpos);
    }

    // HTML to plain text
    $body = drupal_html_to_text($body);

    $file_text = '';
    if(count($files)){
      $file_text = "\n\nFiles:\n";
      foreach($files as $file){
        $file = (array)$file;
        $file_text .= file_create_url($file['uri']) . "\n";
      }
    }

    $reply_text = "(REPLY ABOVE THIS LINE)\n\n";
    $separator = "\n\n-----------------------------------------\n";
    $reply_text_2 = "You can reply to this email and your comments will be added to the web forum.\n";
    //use path alias if available
    $link_url = drupal_get_path_alias('node/' . $nid);
    $join_text = "Or you can join this discussion directly at this link:\n" . $base_url . "/" . $link_url . $comment_text . "\n\n";
    $settings_text = "Too many emails? You can change to Daily or Weekly Digests here:\n" . $base_url . "/user/me/femail";

    $body = $reply_text . $body . $file_text . $separator . $reply_text_2 . $join_text . $settings_text;

    $shortname = $from_emails[$tid];
    $forum_email = $shortname . '@' . $parts['host'];
    $message = array(
      'id' => 'femail_message',
      'from' => $from,
      'subject' => "[" . $shortname . "] " . $node->title,
      'body' => drupal_wrap_mail($body),
      'to' => $forum_email,
      'headers' => array(
        'Return-path' => $forum_email,
        'From' => $from,
        'Message-id' => $msgid,
        'Reply-to' => $forum_email,
        'List-id' => check_plain($forum_term->name) . ' <' . str_replace("@", ".", $forum_email) . '>',
        'List-post' => '<mailto:' . $forum_email . '>',
        'List-archive' => '<' . url('forum/' . $tid, array(
            'absolute' => TRUE
          )) . '>',
        'Thread-topic' => check_plain($node->title),
        'List-subscribe' => '<' . url('user/register', array(
            'absolute' => TRUE
          )) . '>',
        'Bcc' => implode(", ", $emails)
      )
    );
    if($inreplyto){
      $message['headers']['In-reply-to'] = $inreplyto;
    }

    // Give other modules a change to alter the message before sending.
    $context = array(
      'node' => $node,
      'forum' => $forum_term,
      'msgid' => $msgid,
    );
    drupal_alter('femail_message', $message, $context);

    $system = drupal_mail_system('femail', $msgid);
    $system->mail($message);
  }
}

function _femail_send_digest($last_digest, $digest = 2){
  global $base_url;
  $parts = parse_url($base_url);

  // If weekly digest, subtract 6 days from daily digest
  if ($digest == 3) {
    $last_digest = $last_digest - (60*60*24*6);
  }
  // Set digest type string
  if ($digest == 2) {
    $digest_type = 'daily';
  }
  else {
    $digest_type = 'weekly';
  }
  $site_name = variable_get('site_name');
  $site_email = variable_get('site_mail', '');
  $subject = $site_name . ' ' . ucwords($digest_type) . ' Digest';
  $body = array(
    'header' => '',
    'table_of_contents' => array(
      'header' => '',
      'forums' => array(),
      'footer' => ''
    ),
    'footer' => '',
    'content' => array(
      'header' => '',
      'forums' => array(),
      'footer' => ''
    )
  );
  // Setup array of user emails to store their digest while it is generated and then send all at once at the end
  $user_digest_emails = _femail_get_all_emails($digest);
  $user_emails_content_array = array();
  foreach($user_digest_emails as $user_digest_email){
    $user_emails_content_array[$user_digest_email] = array();
  }
  foreach($user_emails_content_array as &$user_email_content){
    $user_email_content['table_of_contents'] = '';
    $user_email_content['message_body'] = '';
  }
  // Create the message HTML sections.
  $body['header'] = '<!DOCTYPE html>';
  $body['header'] .= '<html><body>';
  $body['header'] .= '<h1>' . $site_name . ' ' . ucwords($digest_type) . ' Digest' . '</h1>';
  $body['header'] .= '<p>You asked to be notified on a ' . $digest_type . ' basis of any new CJP forum activity.</p>';
  $body['header'] .= '<h2>Table of Contents</h2>';
  $body['header'] .= '<a href="#skip">Skip to content</a><br><br>';
  $body['header'] .= '<table><tr><td>';

  $body['table_of_contents']['header'] .= '<table id="table_of_contents" cellspacing="0" cellpadding="5" border="2">';
  $body['table_of_contents']['header'] .= '<tr>';
  $table_of_contents_header_titles = array(
    'Jump to',
    'Forum',
    'Topic',
    'Author',
    'Date'
  );
  foreach($table_of_contents_header_titles as $header_title){
    $body['table_of_contents']['header'] .= '<th style="border-right: 1px solid #333;">';
    $body['table_of_contents']['header'] .= $header_title;
    $body['table_of_contents']['header'] .= '</th>';
  }
  $body['table_of_contents']['header'] .= '</tr>';
  $body['table_of_contents']['footer'] .= '</table>';

  $body['content']['header'] = '<a name="skip">&nbsp;</a>';
  $body['content']['header'] .= '<table id="content" cellspacing="0" cellpadding="0" style="width: 100%;"><tr><td>';
  $body['content']['footer'] = '</td></tr></table>';

  $body['footer'] .= '</td></tr></table>';
  $body['footer'] .= '<table>';
  $body['footer'] .= '<tr>';
  $body['footer'] .= '<td>';
  $body['footer'] .= '<p>';
  $body['footer'] .= 'This digest was published for you on ' . date('l F jS, Y \a\t g:i a', time()) . '.';
  $body['footer'] .= '&nbsp;This digest is being sent to registered members of ' . $site_name . ' forums. You can change or delete your subscription from the <a href="' . $base_url . '/user/me/femail">User Control Panel</a>.';
  $body['footer'] .= '&nbsp;If you have questions or feedback on the format of this digest please send it to the <a href="mailto:' . $site_email . '">' . $site_name .' Webmaster</a>.';
  $body['footer'] .= '</p>';
  $body['footer'] .= '<p>';
  $body['footer'] .= 'Powered by <a href="https://www.drupal.org/drupal-7.0">Drupal 7</a> using the <a href="https://www.drupal.org/project/femail">Forum Email Integration Module</a>.';
  $body['footer'] .= '</p>';
  $body['footer'] .= '</td>';
  $body['footer'] .= '</tr>';
  $body['footer'] .= '</table>';
  $body['footer'] .= '</body></html>';
  $td_open_tag = '<td>';

  // Iterate through all forums.
  $all_forums_result = _femail_select_all_forums();
  foreach($all_forums_result as $forum_record) {
    // Load Forum term.
    $tid = $forum_record->tid;
    $body['table_of_contents']['forums'][$tid] = array();
    $body['content']['forums'][$tid] = array();
    $forum_term = _femail_load_forum_term($tid);

    // Get new topics.
    $new_topics_result = _femail_load_new_topics($tid, $last_digest);
    $new_topic = 0;
    foreach($new_topics_result as $new_topic_record) {
      // Start table row.
      $row = '<tr>';
      // Jump to
      $row .= $td_open_tag;
      $row .= '<a href="#node-' . $new_topic_record->nid . '">' . $new_topic_record->nid . '</a>';
      $row .= '</td>';
      // Forum
      $row .= $td_open_tag;
      $row .= $forum_term['name'];
      $row .= '</td>';
      // Topic
      $row .= $td_open_tag;
      $row .= $new_topic_record->title;
      $row .= '</td>';
      // Author
      $node = node_load($new_topic_record->nid);
      $author_account = user_load($node->uid);
      $author_name = $author_account->name;
      $row .= $td_open_tag;
      $row .= $author_name;
      $row .= '</td>';
      // Date
      $row .= '<td>';
      $row .= date('D M j, Y h:i a', $new_topic_record->last_comment_timestamp);
      $row .= '</td>';
      // End table row.
      $row .= '</tr>';

      $body['table_of_contents']['forums'][$tid][$new_topic_record->nid] = $row;
      $body['content']['forums'][$tid][$new_topic_record->nid] = array();

      // Fetch all users who commented.
      $post_commenters_result = _femail_load_post_commenters($new_topic_record->nid);
      // Comment count for each user.
      foreach($post_commenters_result as $post_commenter_record) {
        $user_comments_result = _femail_load_user_comments($new_topic_record->nid, $post_commenter_record->uid);
        foreach($user_comments_result as $user_comment_record){
          $comment_user = $user_comment_record->name;
          // Start table row.
          $row = '<tr>';
          // Jump to
          $row .= $td_open_tag;
          $anchor_comment_link = '#node-' . $new_topic_record->nid . '-comment-' . $user_comment_record->cid;
          $anchor_comment_link_title = $new_topic_record->nid . '#' . $user_comment_record->cid;
          $row .= '<a href="' . $anchor_comment_link . '">' . $anchor_comment_link_title . '</a>';
          $row .= '</td>';
          // Forum
          $row .= $td_open_tag;
          $row .= $forum_term['name'];
          $row .= '</td>';
          // Topic
          $row .= $td_open_tag;
          $row .= $new_topic_record->title;
          $row .= '</td>';
          // Author
          $row .= $td_open_tag;
          $row .= $comment_user;
          $row .= '</td>';
          // Date
          $row .= '<td style="border-top: 1px solid #333;">';
          $row .= date('D M j, Y h:i a', $new_topic_record->last_comment_timestamp);
          $row .= '</td>';
          // End table row.
          $row .= '</tr>';

          $body['table_of_contents']['forums'][$tid][] = $row;
          $body['content']['forums'][$tid][$new_topic_record->nid][$user_comment_record->cid] = '';
        }
      }
      $new_topic++;
    }

    // Assemble HTML pieces
    $toc_body = _femail_assemble_toc_section($body, $tid);
    $message_body = _femail_assemble_message_body_section($body, $tid);
    // Get this forums digest subscribers but not additional emails
    $emails = _femail_get_emails($tid, 0, $digest, FALSE);
    // Remove <> around emails
    foreach($emails as &$email){
      $email = preg_replace('/[<>]/', '', $email);
    }
    // Add HTML parts to users
    foreach($emails as $digest_subscriber_email){
      $user_emails_content_array[$digest_subscriber_email]['table_of_contents'] .= $toc_body;
      $user_emails_content_array[$digest_subscriber_email]['message_body'] .= $message_body;
    }
  }
  // Send Digest if it has users to send to
  if(count($user_emails_content_array)){
    $msgid = '<' . md5(microtime()) . '@' . $parts['host'] . '>';

    // Send mail to digest subscribers
    $message = array(
      'id' => 'femail_message',
      'from' => 'digest@' . $parts['host'],
      'headers' => array(
        'Return-path' => 'digest@' . $parts['host'],
        'From' => 'digest@' . $parts['host'],
        'Message-id' => $msgid,
        'Reply-to' => 'digest@' . $parts['host'],
        'List-post' => '<mailto:' . 'digest@' . $parts['host'] . '>',
        'List-subscribe' => '<' . url('user/register', array(
            'absolute' => TRUE
          )) . '>',
      )
    );
    // Send message to each user & their additional emails as CCs
    foreach($user_emails_content_array as $user_email => $user_message_data){
      // Only send to users that have updates in the forums they are subscribed to
      if($user_message_data['table_of_contents'] != ''){
        $account = user_load_by_mail($user_email);
        // Add additional emails as CCs
        $additional_emails = _femail_load_additional_verified_emails($account->uid);
        $additional_emails_array = array();
        foreach($additional_emails as $additional_email){
          $additional_emails_array[] = '<' . $additional_email->email . '>';
        }
        $message['to'] = '<' . $user_email . '>';
        $message['headers']['Cc'] = implode(", ", $additional_emails_array);
        $message['body'] = _femail_assemble_user_message($body, $user_message_data, $user_email);
        /*        $debug = array(
                  '$user_email' => $user_email,
                  '$account' => $account,
                  '$user_emails_content_array' => $user_emails_content_array
                );
                $message['body'] .= print_r($debug, true);*/
        $message['subject'] = $subject . ' for ' . $account->name;
        // Give other modules a chance to alter the message before sending.
        $context = array(
          'forum' => $forum_term,
          'msgid' => $msgid,
        );
        drupal_alter('femail_digest', $message, $context);
        $system = drupal_mail_system('femail', $msgid);
        $system->mail($message);
      }
    }
  }
}

function _femail_select_all_forums(){
  $query = db_select('forum_index', 'f');
  $query->fields('f', array('tid'));
  $query->distinct();
  $all_forums_result = $query->execute();
  return $all_forums_result;
}

function _femail_load_forum_term($tid){
  $query = db_select('taxonomy_term_data', 't');
  $query->condition('t.tid', $tid, '=');
  $query->fields('t', array('tid', 'name'));
  $forum_term_result = $query->execute()->fetchAssoc();
  return $forum_term_result;
}

function _femail_load_new_topics($tid, $last_digest){
  $query = db_select('forum_index', 'f');
  $query->fields('f', array('title', 'nid', 'last_comment_timestamp'));
  $query->condition('tid', $tid);
  $query->condition('last_comment_timestamp', $last_digest, '>');
  $query->orderBy('created', 'DESC');
  $new_topics_result = $query->execute();
  return $new_topics_result;
}

function _femail_load_post_commenters($nid){
  $query = db_select('comment', 'c');
  $query->fields('c', array('uid', 'name'));
  $query->condition('nid', $nid);
  $query->distinct();
  $post_commenters_result = $query->execute();
  return $post_commenters_result;
}

function _femail_load_user_comments($nid, $uid){
  $query = db_select('comment', 'c');
  $query->fields('c', array('uid', 'name', 'cid'));
  $query->condition('nid', $nid);
  $query->condition('uid', $uid);
  $user_comment_result = $query->execute();
  return $user_comment_result;
}

function _femail_assemble_toc_section($body, $tid){
  // Table of contents section
  $html_body = '';
  foreach($body['table_of_contents']['forums'][$tid] as $toc_post){
    $html_body .= $toc_post;
  }
  return $html_body;
}

function _femail_assemble_message_body_section($body, $tid){
  global $base_url;
  $date_format = 'D M j, Y g:i a';
  // Forum posts/message body
  $html_body = '';
  if(empty($body['content']['forums'][$tid])){
    return $html_body;
  }
  // Forum taxonomy title
  $forum_term = taxonomy_term_load($tid);
//  $html_body .= '<table style="border-color: #333; border-style: solid; border-width: 2px; width: 100%;" cellspacing="0" cellpadding="5">';
  $html_body .= '<table border="1" cellspacing="0" cellpadding="5" style="width:100%;">';
  $html_body .= '<tr>';
  $forum_link = drupal_get_path_alias('forum/' . $tid);
  $html_body .= '<th>';
  $html_body .= 'Forum: ' . '<a href="' . $base_url . '/' . $forum_link . '">' . $forum_term->name . '</a>';
  $html_body .= '</th>';
  $html_body .= '</tr>';

  // Forum posts
  foreach($body['content']['forums'][$tid] as $post_id => $forum_post) {
    $html_body .= '<tr>';
    $html_body .= '<td>';
    $forum_post_node = node_load($post_id);
    if(
      isset($forum_post_node->body[LANGUAGE_NONE])
      &&
      !empty($forum_post_node->body[LANGUAGE_NONE])
      &&
      $forum_post_node->body[LANGUAGE_NONE][0]['value'] != ''
    ){
      // Forum topic.
      $html_body .= '<table border="1" cellspacing="0" cellpadding="5" style="width:100%;">';
      $html_body .= '<tr>';
      $forum_post_link = drupal_get_path_alias('node/' . $forum_post_node->nid);
      $forum_post_link = $base_url . '/' . $forum_post_link;
      $html_body .= '<th>';
      $html_body .= 'Topic: ' . '<a href="' . $forum_post_link . '">' . $forum_post_node->title . '</a>';
      $html_body .= '</th>';
      $html_body .= '</tr>';
      // Forum post.
      $html_body .= '<tr>';
      $html_body .= '<td>';
      $forum_post_anchor_link = '<a name="node-' . $forum_post_node->nid . '">&nbsp;</a>';
      $html_body .= '<h4>' . $forum_post_anchor_link . 'Post: ' . '<a href="' . $forum_post_link . '">' . $forum_post_node->title . '</a>' . '</h4>';
      $forum_post_user = user_load($forum_post_node->uid);
      $forum_post_user_link = $base_url . '/' . drupal_get_path_alias('user/' . $forum_post_node->uid);
      $html_body .= '<p>';
      $html_body .= 'By: ' . '<a href="' . $forum_post_user_link . '">' . $forum_post_user->name . '</a>';
      $html_body .= ' on ' . date($date_format, $forum_post_node->changed);
      $html_body .= '</p>';
      $html_body .= check_markup($forum_post_node->body[LANGUAGE_NONE][0]['value'], 'full_html');
      // Comment(s).
      foreach ($forum_post as $comment_id => $comment) {
        $comment_entity = comment_load($comment_id);
        if(
          isset($comment_entity->comment_body[LANGUAGE_NONE])
          &&
          !empty($comment_entity->comment_body[LANGUAGE_NONE])
          &&
          $comment_entity->comment_body[LANGUAGE_NONE][0]['value'] != ''
        ){
          $comment_entity_user = user_load($comment_entity->uid);
          $comment_anchor_link = '<a name="node-' . $forum_post_node->nid . '-comment-' . $comment_id . '">&nbsp;</a>';
          // Comment link - use path alias if available.
          $comment_link = drupal_get_path_alias('node/' . $forum_post_node->nid);
          $comment_link = $base_url . '/' . $comment_link .  '#comment-' . $comment_id;
          $html_body .= '<h4>' . $comment_anchor_link . 'Comment: ' . '<a href="' . $comment_link . '">' . $comment_entity->subject . '</a>' . '</h4>';
          // User link - use path alias if available.
          $comment_user_link = $base_url . '/' . drupal_get_path_alias('user/' . $comment_entity->uid);
          $html_body .= '<p>';
          $html_body .= 'By: ' . '<a href="' . $comment_user_link . '">' . $comment_entity_user->name . '</a>';
          $html_body .= ' on ' . date($date_format, $comment_entity->changed);
          $html_body .= '</p>';
          $html_body .= check_markup($comment_entity->comment_body[LANGUAGE_NONE][0]['value'], 'full_html');
        }
      }
      $html_body .= '<p style="font-weight: bold;">';
      $html_body .= 'Reply: <a href="' . $forum_post_link . '#comment-form' . '">' . $forum_post_node->title . '</a>';
      $html_body .= '</p>';

      $html_body .= '</td>';
      $html_body .= '</tr>';
      $html_body .= '</table>';
    }
  }
  $html_body .= '</td>';
  $html_body .= '</tr>';
  $html_body .= '</table>';
  return $html_body;
}

function _femail_check_user_digest_option($uid, $tid){
  $query = db_select('femail_digest', 'f');
  $query->condition('uid', $uid);
  $query->condition('tid', $tid);
  $query->fields('f', array('digest'));
  $user_digest = $query->execute()->fetchField();
  return $user_digest;
}

function _femail_check_forum_override($tid){
  $query = db_select('femail_override', 'f');
  $query->condition('tid', $tid);
  $query->fields('f', array('override'));
  $forum_override = $query->execute()->fetchField();
  return $forum_override;
}

function _femail_load_additional_verified_emails($uid){
  $additional_emails = db_select('femail_user_emails', 'f')
    ->fields('f', array('email'))
    ->condition('uid', $uid)
    ->condition('status', '1')
    ->condition('enabled', '1')
    ->execute()->fetchAll();
  return $additional_emails;
}

function _femail_load_user_copy($uid){
  $user_email_copy = db_select('femail_user_email_copy', 'f')
    ->condition('uid', $uid)
    ->fields('f', array('email_copy'))
    ->execute()
    ->fetch();

  return is_object($user_email_copy)? $user_email_copy->email_copy : FALSE;
}

function _femail_select_active_users($user_email_copy, $uid){
  $query = db_select('users', 'u');
  $query->condition('u.status', 0, '!=');
  // Allow users with the user email copy value to receive a copy of their own emails.
  if(!$user_email_copy){
    $query->condition('u.uid', $uid, '!=');
  }
  $query->fields('u', array('uid'));
  $query->distinct(TRUE);
  $results = $query->execute();
  return $results;
}

function _femail_assemble_user_message($body, $user_message_data, $user_email){
  $messageHTML = $body['header'];
  $messageHTML .= $body['table_of_contents']['header'];
  $messageHTML .= $user_message_data['table_of_contents'];
  $messageHTML .= $body['table_of_contents']['footer'];
  $messageHTML .= $body['content']['header'];
  $messageHTML .= $user_message_data['message_body'];
  $messageHTML .= $body['content']['footer'];
  $messageHTML .= $body['footer'];

  return $messageHTML;
}