<?php

/**
 * Presents a user with a digest form on their account page.
 * Amended to include the second option from femail to add additional email addresses to an account
 */
function femail_user_settings($account){
  // Title of the page is the user's name
  drupal_set_title(check_plain($account->name));
  if(!user_access('post by femail', $account)){
    drupal_set_message(t('This user account can not post messages to the forum by email'), 'error');
  }
  // ---------------- Digest form --------------------------------------
  // $output = rendered HTML
  $output = theme('femail_box', array(
    'title' => t('Forum Email Settings'),
    'content' => drupal_render(drupal_get_form('femail_user_subscribe', $account))
  ));
  // ---------------- Add additional emails ------------------------------------
  // Table for additional emails
  $header = array(
    t('Email'),
    t('Status'),
    t('Enabled'),
    t('Operations')
  );
  $rows = array();
  $results = db_select('femail_user_emails', 'f')->condition('uid', $account->uid)->fields('f', array(
    'email_id',
    'status',
    'enabled',
    'email'
  ))->execute();
  // List each of the email addresses associated with the account as a row.
  foreach($results as $mail_bits){
    $status = $mail_bits->status ? t('Verified') : t('Not verified');
    $enabled = $mail_bits->enabled ? t('Enabled') : t('Disabled');
    $enabled_link_options = array(
      'attributes' => array(
        'title' => 'Click to ' . ($mail_bits->enabled ? 'Disable' : 'Enable')
      )
    );
    $enabled_link = 'femail/use-mail/' . $account->uid . '/' . $mail_bits->email_id . '/' . (!$mail_bits->enabled ? '1' : '0');
    $rows[] = array(
      check_plain($mail_bits->email),
      $status,
      l($enabled, $enabled_link, $enabled_link_options),
      l(t('Remove'), 'femail/remove/' . $account->uid . '/' . $mail_bits->email)
    );
  }
  // Theme the table, along with the form
  $output .= theme('femail_box_additional_emails', array(
    'title' => t('Your additional email addresses'),
    'content' => theme('table', array(
        'header' => $header,
        'rows' => $rows
      )) . drupal_render(drupal_get_form('femail_user_add', $account))
  ));
  // Return both forms
  return $output;
}

/**
 * Simple replacement for Drupal 6 theme_box "theme('box', boxy shit)".
 *
 * Interestingly, there is still an issue open on Drupal.org explaining that
 * they should document the fact this theme was removed.
 */
function theme_femail_box($variables){
  $description = 'In this section, you are able to set your "Receive Own Posts" setting and you may set the frequency of email notifications from each Forum. ';
  $description .= 'The "Receive Own Posts" setting allows you to receive a copy of posts you make, which is frequently referred to as the "Acknowledge" setting. ';
  $description .= 'Daily Digests are delivered near midnight and Weekly Digests are delivered early Mondays.';
  $output = '<h2 class="title">' . $variables['title'] . '</h2>';
  $output .= '<div class="description">'. t($description) . '</div>';
  $output .= '<div>' . $variables['content'] . '</div>';
  return $output;
}

/**
 * Simple replacement for Drupal 6 theme_box "theme('box', boxy shit)".
 *
 * Interestingly, there is still an issue open on Drupal.org explaining that
 * they should document the fact this theme was removed.
 *
 * themes the 'additional emails' section of the user options form
 */
function theme_femail_box_additional_emails($variables){
  $output = '<h2 class="title">' . $variables['title'] . '</h2>';
  $output .= '<div style="padding-top:10px;padding-bottom:5px">'. t("Emails in this section will receive a copy of posts made to forums your account is subscribed to above as well as be able to post via email to those forums.") . '</div>';
  $output .= '<div>' . $variables['content'] . '</div>';
  return $output;
}

/**
 * Simple callback that will verify an email.  The link is intended to be
 * clicked on from an email, so it's fine to redirect to the front page.
 * Verify an email
 */
function femail_email_verify($email, $hash){
  // Check the hash is correct.
  if(md5($email . variable_get('femail_install_time', 'INSTALLTIME')) == $hash){
    // Correct - we should mark this email as verified.
    db_update('femail_user_emails')->condition('email', $email)->fields(array('status' => 1))->execute();
    drupal_set_message(t('Email verified'));
  }else{
    // Hash failed - we'll add an error message and log it to the watchdog.
    watchdog('femail', 'Verification of an email address failed. Details: !ipaddress , !email , !hash', array(
      '!ipaddress' => ip_address(),
      '!email' => $email,
      '!hash' => $hash
    ), WATCHDOG_NOTICE);
    drupal_set_message(t('Email verification failed'));
  }
  drupal_goto();
}

/**
 * FIXME - This needs to be improved to ask the user "Are you sure?"
 *
 * This currently removes an email address from the femail_user_emails table
 * without asking for any approval.  This could no doubt result in potential
 * issues.
 */
function femail_email_remove($account, $email){
  // If we're here, we just remove
  db_delete('femail_user_emails')->condition('email', $email)->condition('uid', $account->uid)->execute();
  drupal_set_message("Email removed");
  drupal_goto('user/' . $account->uid . '/femail');
}

/**
 * Form for selecting which forums to subscribe to
 */
function femail_user_subscribe($form, $form_state){
  // User Account
  $uid = $form_state['build_info']['args'][0]->uid;
  $user = user_load($uid);

  // It's OK to use "taxonomy_get_tree", as NO FORUM IS HUGE!
  $forum_tree = taxonomy_get_tree(variable_get('forum_nav_vocabulary', 0));
  $forum_containers = variable_get('forum_containers', array());

  //email copy (ACK/acknowledge) settings - get a copy of your own posts
  $email_copy_result = db_select('femail_user_email_copy', 'fuem')
    ->fields('fuem', array('email_copy'))
    ->condition('uid', $uid, '=')
    ->execute()
    ->fetchAssoc();
  if($email_copy_result === FALSE){
    $email_copy_value = 0;
  }
  else{
    $email_copy_value = $email_copy_result['email_copy'];
  }
  $form['femail_receive_own_posts'] = array(
    '#type' => 'checkbox',
    '#title' => t('Receive Own Posts'),
    '#default_value' => $email_copy_value
  );
  //
  $form['femail_digest_quicktoggle'] = array(
    '#type' => 'radios',
    '#title' => 'Forum Email Frequency Quick Toggle',
    '#description' => 'Quickly set all forums to a single receipt frequency setting.',
    '#options' => array(0 => 'None', 1 => 'Immediate', 2 => 'Daily', 3 => 'Weekly'),
    '#id' => 'femail-digest-quicktoggle'
  );
  $form['submit_above'] = array(
    '#type' => 'submit',
    '#value' => t('Save Options'),
    '#id' => 'submit-above',
    '#attributes' => array(
      'class' => array(
        'btn-lg'
      )
    )
  );
  // Digest settings
  $form['femail_digest'] = array(
    '#tree' => TRUE,
    '#prefix' => '<table id="femail_digest" class="table table-striped"><tr class="heading"><th>' . t('Forum Name') . '</th><th>' . t('Email Notification Frequency') . '</th><tr>',
    '#suffix' => '</table>'
  );

  $class = "odd";
  foreach($forum_tree as $branch){
    $user_access = forum_access_access('create', $branch->tid, $user);

    if ($user_access) {
      if($class == "odd"){
        $class = "even";
      }else{
        $class = "odd";
      }
      if(in_array($branch->tid, $forum_containers)){
        // Container
        $form['femail_digest'][$branch->tid] = array(
          '#markup' => '<tr class="' . $class . ' forum-section"><td colspan="2">' . check_plain($branch->name) . ' Forums' . '</td></tr>'
        );
      }else{
        // Forum

        // User Settings
        $query = db_select('femail_digest', 'f');
        $query->condition('tid', $branch->tid);
        $query->condition('uid', $user->uid);
        $query->fields('f', array('digest'));
        $digest = $query->execute()->fetchField();
        if ($digest == NULL) {
          $digest = 1;
        }

        // Forum Override
        $query = db_select('femail_override', 'f');
        $query->condition('tid', $branch->tid);
        $query->fields('f', array('override'));
        $override = $query->execute()->fetchField();
        if ($override == NULL) {
          $override = -1;
        }

        $value = ($override >= 0) ? $override : $digest;
        $disabled = ($override >= 0) ? 1 : 0;

        $form['femail_digest'][$branch->tid] = array(
          '#type' => 'item',
          '#title' => check_plain($branch->name),
          '#prefix' => '<tr class="' . $class . '"><td style="padding-left:' . (($branch->depth * 10) + 5) . 'px">',
          '#suffix' => '</td>'
        );
        $form['femail_digest'][$branch->tid][0] = array(
          '#type' => 'radios',
          '#disabled' => $disabled,
          '#default_value' => $value,
          '#options' => array(0 => 'None', 1 => 'Immediate', 2 => 'Daily', 3 => 'Weekly'),
          '#prefix' => '<td class="femail-radios">',
          '#suffix' => '</td></tr>'
        );
      }
    }
  }
  // Hidden field for the UID - just makes things simpler later on.
  $form['femail_uid'] = array(
    '#type' => 'hidden',
    '#value' => $form_state['build_info']['args'][0]->uid
  );
  $form['submit_below'] = array(
    '#type' => 'submit',
    '#value' => t('Save Options'),
    '#id' => 'submit-below',
    '#attributes' => array(
      'class' => array(
        'btn-lg'
      )
    )
  );
  return $form;
}

/**
 * Submit function for a user's Femail subscriptions.
 */
function femail_user_subscribe_submit($form, $form_state){
  // email_copy/receive own posts setting
  db_merge('femail_user_email_copy')
    ->key(array('uid' => $form_state['values']['femail_uid']))
    ->fields(array(
      'uid' => $form_state['values']['femail_uid'],
      'email_copy' => $form_state['values']['femail_receive_own_posts']
    ))->execute();
  // digest/subscription settings
  // Lets delete everything from the femail_digest for this user
  $query = db_delete('femail_digest')->condition('uid', $form_state['values']['femail_uid'])->execute();
  // loop through values and insert each forum's setting
  foreach($form_state['values']['femail_digest'] as $tid => $value){
    db_insert('femail_digest')->fields(array(
      'uid' => $form_state['values']['femail_uid'],
      'tid' => $tid,
      'digest' => $value
    ))->execute();
  }
  drupal_set_message("Settings updated");
}

/**
 * Provides the form for adding an additional email address for a user.
 */
function femail_user_add($form, $form_state){
  $account = $form_state['build_info']['args'][0];
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Add Additional Email')
  );
  $form['femail_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Email')
  );
  $form['femail_uid'] = array(
    '#type' => 'hidden',
    '#value' => $account->uid
  );
  return $form;
}

/**
 * Submit function that adds an email address for a user.  This will send out
 * the email to the user so that they can validate their email.
 */
function femail_user_add_submit($form, $form_state){
  $account = user_load($form_state['values']['femail_uid']);
  drupal_mail('femail', 'verify', $form_state['values']['femail_email'], user_preferred_language($account));
  db_insert('femail_user_emails')->fields(array(
    'uid' => $form_state['values']['femail_uid'],
    'email' => $form_state['values']['femail_email'],
    'status' => 0
  ))->execute();
  drupal_set_message($form_state['values']['femail_email'] . " added. Please now check your email to validate this address.");
}

/**
 * Ensure that the email supplied is not already in use on the site.  Also
 * ensure that the email address is a valid email.
 */
function femail_user_add_validate($form, &$form_state){
  // Check that it is a valid email address
  if(!valid_email_address($form_state['values']['femail_email'])){
    form_set_error('femail_email', t('Email address provided is not valid'));
  }
  // Check for existing entries.
  $mails = db_select('femail_user_emails', 'f')->fields('f', array(
    'email'
  ))->condition('email', $form_state['values']['femail_email'])->union(db_select('users', 'u')->fields('u', array(
    'mail'
  ))->condition('mail', $form_state['values']['femail_email']))->execute()->fetchField();
  if($mails){
    form_set_error('femail_email', t('Email is already in use on this site.'));
  }
}

/** toggles use of additional email and redirects back to femail user settings page */
function femail_additional_email_toggle_use($account, $email_id, $use_mail_boolean){
  // If we're here, we just toggle and redirect
  $updated = db_update('femail_user_emails') // Table name no longer needs {}
  ->fields(array(
    'enabled' => $use_mail_boolean,
  ))
    ->condition('email_id', $email_id)
    ->execute();

  drupal_set_message('Email set as ' . ($use_mail_boolean ? 'enabled' : 'disabled'));
  drupal_goto('user/' . $account->uid . '/femail');
}
